
//#define _POSIX_C_SOURCE 200809L


#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

// RTEMS configuration
#define CONFIGURE_INIT
#include "init.h"

#include <bsp.h>
#include <libcpu/stackTrace.h>
#include <rtems.h>

#include <errno.h>
#include <inttypes.h>
#include <stdio.h>
#include <string.h>
#include <time.h>


void *init(void *args)
{
	struct timespec lasttime = { 0 };

	(void) args; // Unused


	printf("Starting, %" PRIu32 " ticks/s...\n", (uint32_t) rtems_clock_get_ticks_per_second());


	while (1)
	{
		struct timespec newtime;

		if (clock_gettime(CLOCK_MONOTONIC, &newtime) != 0)
		{
			fprintf(stderr, "clock_gettime() failed: \"%s\"\n", strerror(errno));

			return (NULL);
		}


		if (!(newtime.tv_sec > lasttime.tv_sec
		      || (newtime.tv_sec == lasttime.tv_sec
		          && newtime.tv_nsec >= lasttime.tv_nsec)))
		{
			printf("Time went backwards, from {%ld, %ld} to {%ld, %ld}\n", lasttime.tv_sec, lasttime.tv_nsec, newtime.tv_sec, newtime.tv_nsec);
		}
//		else
//			printf("Time from {%ld, %ld} to {%ld, %ld}\n", lasttime.tv_sec, lasttime.tv_nsec, newtime.tv_sec, newtime.tv_nsec);


		lasttime = newtime;
	}


	pthread_exit(NULL);
	return (NULL);
}


void fatal_extension(Internal_errors_Source the_source,
                     bool                   is_internal,
                     Internal_errors_t      the_error)
{
	uint32_t isr_cookie;


	fflush(stdout);

	printk("Fatal %sinternal error: Source: %u, error: %u. Stack trace:\n", is_internal ? "" : "non-", the_source, the_error);
	CPU_print_stack(); // RTEMS-on-PowerPC-specific


	// We can't call _CPU_Fatal_halt() as it'll end up calling us again, so halt ourselves
	//! \bug Not SMP safe
	_CPU_ISR_Disable(isr_cookie);

	(void) isr_cookie; // Shut GCC up about isr_cookie being set but not used. We don't care


	while (1)
		;
}
